searchData={"items":[{"type":"module","title":"WaitForIt","doc":"Provides various ways of waiting for things to happen.","ref":"WaitForIt.html"},{"type":"module","title":"Overview - WaitForIt","doc":"Elixir is a functional programming language with an emphasis on immmutability of data. However,\nwhen dealing with shared state or interacting with external systems, *change happens*.\n\nWaitForIt provides various ways of waiting for such changes to happen.\n\nWhile Elixir provides several language and standard library features (such as\n`Process.sleep/1`, `receive/1`/`after`, and `Task.async/1`/`Task.await/2`) that can be used to\nimplement waiting, they are inconvenient to use for this purpose. WaitForIt builds on top of\nthese language features to provide convenient and easy-to-use facilities for waiting on specific\nconditions. While this is likely most useful for test code in which tests must wait for\nconcurrent or asynchronous activities to complete, it is also useful in any scenario where\nconcurrent processes must coordinate their activity. Examples include asynchronous event\nhandling, producer-consumer processes, and time-based activity.","ref":"WaitForIt.html#module-overview"},{"type":"module","title":"Quick start - WaitForIt","doc":"To use WaitForIt, you must first `require WaitForIt` or `import WaitForIt`.\n\nThere are three distinct forms of waiting provided. Jump to the docs for each for more\ninformation.\n\n##","ref":"WaitForIt.html#module-quick-start"},{"type":"module","title":"wait - WaitForIt","doc":"The `wait/2` macro waits until a given expression evaluates to a truthy value.\n\n    # Wait up to one minute for a file to exist, and then print its contents\n    if WaitForIt.wait(File.exists?(\"data.csv\"), timeout: :timer.minutes(1)) do\n      IO.puts(File.read!(\"data.csv\"))\n    else\n      IO.warn(\"Stopped waiting for the file to exist\")\n    end\n\n##","ref":"WaitForIt.html#module-wait"},{"type":"module","title":"case_wait - WaitForIt","doc":"The `case_wait/3` macro waits until a given expression evaluates to a value that matches any one\nof the given case clauses. It looks and acts like an Elixir `case/2` expression except that it\ncan take an optional `else` clause.\n\n    # Wait for 30 seconds for a directory to exist, and then write a file in it\n    WaitForIt.case_wait(File.stat(\"data\"), timeout: :timer.seconds(30)) do\n      {:ok, %File.Stat{type: :directory}} ->\n        File.write!(\"data/greeting.txt\", \"Hello, world!\")\n    else\n      {:ok, %File.Stat{type: type}} ->\n        IO.warn(\"Expected 'data' to be a directory but its type is #{inspect(type)}\")\n\n      {:error, reason} ->\n        IO.warn(\"Could not stat 'data': #{inspect(reason)}\")\n    end\n\n##","ref":"WaitForIt.html#module-case_wait"},{"type":"module","title":"cond_wait - WaitForIt","doc":"The `cond_wait/2` macro waits until any one of the given expressions evaluates to a truthy value.\nIt looks and acts like an Elixir `cond/1` expression except that it can take an optional `else`\nclause.\n\n    # Wait for up to one minute for either a specific file to exist OR for the top of the minute\n    # to be reached.\n    WaitForIt.cond_wait(timeout: :timer.seconds(10), frequency: 500) do\n      File.exists?(\"data/process.json\") ->\n        IO.puts(\"Processing...\")\n\n      NaiveDateTime.utc_now().second == 0 ->\n        IO.puts(\"Processing...\")\n    else\n      IO.warn(\"Stopped waiting since neither condition ever became truthy\")\n    end\n\n#","ref":"WaitForIt.html#module-cond_wait"},{"type":"module","title":"Options - WaitForIt","doc":"All three forms of waiting accept the same set of options to control their behavior:\n\n* `:timeout` - the amount of time to wait (in milliseconds) before giving up\n* `:pre_wait` - wait for the given number of milliseconds before evaluating conditions for the first time\n* `:frequency` - the polling frequency (in milliseconds) at which to re-evaluate conditions\n* `:signal` - disable polling and use a signal of the given name instead\n\nSee [Polling-based waiting](#module-polling-based-waiting) for more information on the\n`:frequency` option and [Signal-based waiting](#module-signal-based-waiting) for more\ninformation on the `:signal` option.","ref":"WaitForIt.html#module-options"},{"type":"module","title":"Waitable expressions and waiting conditions - WaitForIt","doc":"_Waitable expressions_ and _waiting conditions_ are fundamental concepts in WaitForIt.\n\nA _waitable expression_ is any arbitrary Elixir expression that can be evaluated one or more\ntimes to produce a value.\n\nA _waiting condition_ is a conditional expression that indicates whether waiting should continue\nor be halted with a particular value.\n\nIn the case of `wait/2`, there is a single waitable expression, which is passed as the first\nargument of the macro, and an implicit waiting condition, which is based on the truthiness\nof the associated waitable expression. For example:\n\n    WaitForIt.wait(2 + 2 == 5, timeout: 200)\n\nIn this example the waitable expression is `2 + 2 == 5` and the implicit waiting condition is\nthe truthiness of that expression. The waitable expression is repeatedly evaluated until the\nvalue that it produces satisfies the waiting condition. In this case, the value of evaluating\nthe expression is always `false` so it will never satisfy the waiting condition of a truthy\nvalue, and will therefore result in a timeout.\n\nFor `case_wait/3`, there is a single waitable expression and one or more explicit waiting\nconditions expressed as case clauses. For example:\n\n    WaitForIt.case_wait(File.stat(\"data.csv\"), timeout: :timer.seconds(10)) do\n      {:ok, %File.Stat{} = file_stat} -> IO.inspect(file_stat)\n    end\n\nIn this example the waitable expression is `File.stat(\"data.csv\")`, which, upon evaluation,\nresults in a value of either `{:ok, %FileStat{}}` or `{:error, reason}`. There is also one\nexplicit waiting condition, which is the case clause `{:ok, %File.Stat{} = file_stat}`.\nThe waitable expression will be repeatedly evaluated until it produces a value satisfying the\nlone waiting condition. In other words, it will wait until the file exists or a timeout occurs.\n\nFor `cond_wait/2`, there can be one or more waitable expressions and each one is paired with an\nimplicit waiting condition, which is the truthiness of the waitable expression value.\nFor example:\n\n    WaitForIt.cond_wait(timeout: :timer.hours(1)) do\n      Date.utc_today().day == 1 -> IO.puts(\"It's the first day of the month\")\n      NaiveDateTime.utc_now().minute == 30 -> IO.puts(\"It's half past the hour\")\n    end\n\nIn this example, there are two waitable expressions: `Date.utc_today().day == 1` and\n`NaiveDateTime.utc_now().minute == 30`. Each of these is paired with an implicit waiting\ncondition, which is the truthiness of the value produced by evaluating the expression.\n\n#","ref":"WaitForIt.html#module-waitable-expressions-and-waiting-conditions"},{"type":"module","title":"Idempotency of waitable expressions - WaitForIt","doc":"Waitable expressions are by their nature subject to change with repeated evaluations over time.\nTherefore, idempotent expressions are of little use in the context of waiting, since waiting\nwould either halt immediately (if the expression already saitisfies the waiting conditions)\nor never halt at all (if it does not satisfy the waiting conditions).\n\nIt is important, however, that any side-effects that can occur during evaluation of the\nexpression are safe and predictable, since the expression may be evaluated an inderminate\nnumber of times while waiting.","ref":"WaitForIt.html#module-idempotency-of-waitable-expressions"},{"type":"module","title":"Polling-based waiting - WaitForIt","doc":"By default, WaitForIt uses a polling-based waiting mode in which waitable expressions are\nperiodically re-evaluated until waiting conditions have been met or a timeout has occurred.\nThe frequency at which waitable expressions are evaluated can be controlled by the `:frequency`\noption, which specifies the delay between evaluations in milliseconds and is supported by all\nforms of waiting.\n\n> ##","ref":"WaitForIt.html#module-polling-based-waiting"},{"type":"module","title":"Polling \"frequency\" {: .neutral} - WaitForIt","doc":">\n> The term \"frequency\" is something of a misnomer as it is used here, since it is a time value\n> (milliseconds) rather than a rate. A more accurate term would be `:polling_interval`, or\n> perhaps simply `:interval`, but `:frequency` is already in use.\n>\n> For the curious, the actual frequency in Hertz can be derived from the value of the\n> `:frequency` option using this formula: `1 / (:frequency / 1000)`\n>\n> Thus a `:frequency` value of 100 yields a frequency of 10 Hz.","ref":"WaitForIt.html#module-polling-frequency-neutral"},{"type":"module","title":"Signal-based waiting - WaitForIt","doc":"Signal-based waiting obviates the need for polling by using a signaling mechanism to indicate\nthat waiting conditions should be re-evaluated in response to some event. With signal-based\nwaiting, instead of periodically re-evaluating conditions at a particular frequency, a signal\nis sent to waiters to indicate when waiting conditions should be re-evaluated. It is expected\nthat the `signal/1` function will be used to unblock the waiting code in order to re-evaluate\nthe waiting conditions.\n\nTo use signal-based waiting instead of polling-based waiting use the `:signal` option that is\nsupported by all forms of waiting. The value of the `:signal` option is an arbitrary term\n(typically an atom or a tuple of atoms) that serves as the binding between the waiting\nconditions and the asynchronous code that can alter the outcome of those waiting conditions.\nWhen the `:signal` option is used, WaitForIt will automatically wait until a matching signal is\nreceived and then re-evaluate waiting conditions. If the waiting conditions are saitisfied then\nthe wait is halted, if not then the wait continues until the next signal is received or a\ntimeout occurs.\n\nBy way of example, imagine a typical producer-consumer problem in which a consumer process waits\nfor items to appear in some buffer while a separate producer process occasionally place items in\nthe buffer. In this scenario, the consumer process might use the `wait/2` macro with the\n`:signal` option to wait until there are some items in the buffer and the producer process would\nuse the `signal/1` function to tell the consumer that it might be time for it to check the\nbuffer again.\n\n    # CONSUMER\n    # assume the existence of a `buffer_size` function\n    WaitForIt.wait buffer_size() >= 4, signal: :wait_for_buffer\n\n    # PRODUCER\n    # put some things in buffer, then:\n    WaitForIt.signal(:wait_for_buffer)\n\nNotice that the same signal name `:wait_for_buffer` is used by both the consumer and the\nproducer, which is what allows the producer to signal to the consumer that waiting conditions\nshould be re-evaluated. It is important to realize that just because a signal has been emitted\ndoes not necessarily mean that any waiting conditions have been satisfied. Rather, a signal\nindicates that waiters should re-evaluate their waiting conditions to determine if they should\ncontinue to wait or not.","ref":"WaitForIt.html#module-signal-based-waiting"},{"type":"module","title":"Using WaitForIt in tests - WaitForIt","doc":"One common use case for waiting on the results of asynchronous operations is in tests,\nparticularly in integration or end-to-end tests. This section will present examples of using\nthe various forms of waiting in test code. All examples assume that the `WaitForIt` module has\nbeen imported in the test module, such as follows:\n\n    defmodule MyTest do\n      use ExUnit.Case\n      import WaitForIt\n    end\n\nThe `wait/2` macro can be used directly in assertions, since it returns the truthy or falsy\nvalue that the waitable expression evaluated to (i.e. a truthy value for successful waits or a\nfalsy value for timeouts). For example, to assert that a particular database record is\neventually inserted into the database can be as simple as:\n\n    assert wait(Repo.get(User, user_id))\n\nAlternatively, pattern-matching can be used in some cases to make stronger assertions, such as:\n\n    assert %User{first_name: \"Elijah\"} = wait(Repo.get(User, user_id), timeout: 1_000)\n\nThe `case_wait/3` macro offers greater flexibility in the sense that it allows for matching on\nany one of a series of case clauses and also allows for the use of an `else` block if none of\nthe case clauses eventually match. For example, to assert that a particular database record is\neventually inserted and that it has particular values:\n\n    case_wait Repo.get(User, user_id), timeout: 1_000 do\n      %User{id: ^user_id} = user ->\n        assert user.first_name == \"Elijah\"\n        assert Date.compare(user.birth_date, ~D[2023-07-20]) == :eq\n    else\n      unexpected ->\n        flunk(\"Expected a User record for Elijah, got something else: #{inspect(unexpected)}\")\n    end\n\nOr to test if exactly one or two records are returned for a particular query, something like\nthe following can be used:\n\n    case_wait Repo.all(some_query), timeout: 2_000, frequency: 500 do\n      [only_thing] -> assert only_thing.id == 42\n      [thing1, thing2] -> assert thing1.id == 1 and thing2.id == 2\n    else\n      [] -> flunk(\"expected one or two things, got no things\")\n      [_ | _] = things -> flunk(\"expected one or two things, got #{length(things)} things\")\n    end","ref":"WaitForIt.html#module-using-waitforit-in-tests"},{"type":"module","title":"A note on \"catch-all\" clauses - WaitForIt","doc":"It is common to include \"catch-all\" clauses in normal Elixir `case/2` and `cond/1` expressions.\nOften, a `case/2` expression will include a final catch-all clause (like `_`) which will always\nmatch, Similarly, a `cond/1` expression will typically include a final always-truthy condition\n(like `true`) which will always match.\n\nWhen using the waiting variants of these constructs, `case_wait/3` and `cond_wait/2`, it is\n*not* recommended to use such catch-all clauses. The reason for this is that, since catch-all\nclauses by definition always match, including one as a waiting condition would not allow for\nre-evaluating any other waiting conditions and would terminate the wait immediately after the\nfirst evaluation.\n\nInstead of using a catch-all clause that always matches, an `else` clause can be used instead.\nBoth `case_wait/3` and `cond_wait/2` support `else` clauses, and these clauses are evaluated\nwhenever a waiting operation results in a timeout, which allows for customizing the behavior\nand return value of the expression in the event of a timeout.","ref":"WaitForIt.html#module-a-note-on-catch-all-clauses"},{"type":"macro","title":"WaitForIt.case_wait/3","doc":"Wait until the given `expression` matches one of the case clauses in the given block.\n\nReturns the value of the matching clause, the value of the optional `else` clause,\nor the last evaluated value of the expression in the event of a timeout.\n\nThe `do` block passed to this macro must be a series of case clauses exactly like a built-in\nElixir `case/2` expression. Just like a `case/2` expression, the clauses will attempt to be\nmatched from top to bottom and the first one that matches will provide the resulting value of the\nexpression. The difference with `case_wait/3` is that if none of the clauses initially matches it\nwill wait and periodically re-evaluate the clauses until one of them does match or a timeout\noccurs.\n\nAn optional `else` clause may also be used to provide the value in case of a timeout. If an\n`else` clause is provided and a timeout occurs, then the `else` clause will be evaluated and\nthe resulting value of the `else` clause becomes the value of the `case_wait/3` expression. If no\n`else` clause is provided and a timeout occurs, then a `CaseClauseError` is raised, exactly as\nif a normal Elixir `case/2` expression were being used.\n\nThe optional `else` clause may also take the form of match clauses, such as those in the `else`\nclause of a `with/1` expression. In this form, the `else` clause can match on the final value\nof the expression that was evaluated before the timeout occurred. See the examples below for an\nexample of this.\n\n> ##","ref":"WaitForIt.html#case_wait/3"},{"type":"macro","title":"Beware \"catch-all\" clauses {: .warning} - WaitForIt.case_wait/3","doc":">\n> `case_wait/3` expressions should *not* include a final \"catch-all\" clause, such as `_`, which\n> would always match. Instead, an `else` clause can be used to customize the behavior and\n> return value in the event of a waiting timeout.\n>\n> See [A note on \"catch-all\" clauses](#module-a-note-on-catch-all-clauses) in the module docs\n> for further information.","ref":"WaitForIt.html#case_wait/3-beware-catch-all-clauses-warning"},{"type":"macro","title":"Options - WaitForIt.case_wait/3","doc":"See the WaitForIt module documentation for further discussion of these options.\n\n  * `:timeout` - the amount of time to wait (in milliseconds) before giving up\n  * `:pre_wait` - wait for the given number of milliseconds before evaluating conditions for the first time\n  * `:frequency` - the polling frequency (in milliseconds) at which to re-evaluate conditions\n  * `:signal` - disable polling and use a signal of the given name instead","ref":"WaitForIt.html#case_wait/3-options"},{"type":"macro","title":"Examples - WaitForIt.case_wait/3","doc":"Wait until queue has at least 5 messages, then return them:\n\n    WaitForIt.case_wait Queue.get_messages(queue), timeout: 30_000, frequency: 100 do\n      messages when length(messages) > 4 -> messages\n    else\n      # If after 30 seconds we still don't have 5 messages, just return the messages we do have.\n      messages -> messages\n    end\n\nA thermostat that keeps temperature in a small range:\n\n    def thermostat(desired_temperature) do\n      WaitForIt.case_wait get_current_temperature() do\n        temp when temp > desired_temperature + 2 ->\n          turn_on_air_conditioning()\n        temp when temp  \n          turn_on_heat()\n      end\n      thermostat(desired_temperature)\n    end\n\nWait until the process mailbox is small enough before flooding it with more messages:\n\n    WaitForIt.case_wait Process.info(stream_pid, :message_queue_len),\n      frequency: 10,\n      timeout: 60_000 do\n      {:message_queue_len, len} when len  \n        send_chunk(conn, chunk)\n    else\n      len ->\n        raise \"Timeout while sending stream response. [message_queue_len: #{len}]\"\n    end\n\n> ##","ref":"WaitForIt.html#case_wait/3-examples"},{"type":"macro","title":"Production-ready {: .info} - WaitForIt.case_wait/3","doc":">\n> The above example is a real-world use of WaitForIt that was used to solve an issue with chunked\n> HTTP responses using [plug_cowboy](https://github.com/elixir-plug/plug_cowboy). The underlying\n> issue has since been fixed but this example is a good illustration of using WaitForIt to\n> solve a production problem.\n>\n> See https://github.com/elixir-plug/plug_cowboy/issues/10 for background and further details,\n> if interested.","ref":"WaitForIt.html#case_wait/3-production-ready-info"},{"type":"macro","title":"WaitForIt.case_wait!/3","doc":"The same as `case_wait/3` but raises a `WaitForIt.TimeoutError` exception if it fails.","ref":"WaitForIt.html#case_wait!/3"},{"type":"macro","title":"WaitForIt.cond_wait/2","doc":"Wait until one of the expressions in the given block evaluates to a truthy value.\n\nReturns the value corresponding with the matching expression, the value of the optional `else`\nclause, or `nil` in the event of a timeout.\n\nThe `do` block passed to this macro must be a series of expressions exactly like a built-in\nElixir `cond/1` expression. Just like a `cond/1` expression, the embedded expresions will be\nevaluated from top to bottom and the first one that is truthy will provide the resulting value of\nthe expression. The difference with `cond_wait/2` is that if none of the expressions is initially\ntruthy it will wait and periodically re-evaluate them until one of them becomes truthy or a\ntimeout occurs.\n\nAn optional `else` clause may also be used to provide the value in case of a timeout. If an\n`else` clause is provided and a timeout occurs, then the `else` clause will be evaluated and\nthe resulting value of the `else` clause becomes the value of the `cond_wait/2` expression. If no\n`else` clause is provided and a timeout occurs, then a `CondClauseError` is raised, exactly as\nif a normal Elixir `cond/1` expression were being used.\n\n> ##","ref":"WaitForIt.html#cond_wait/2"},{"type":"macro","title":"Beware \"catch-all\" clauses {: .warning} - WaitForIt.cond_wait/2","doc":">\n> `cond_wait/2` expressions should *not* include a final \"catch-all\" clause, such as `true`,\n> which would always match. Instead, an `else` clause can be used to customize the behavior and\n> return value in the event of a waiting timeout.\n>\n> See [A note on \"catch-all\" clauses](#module-a-note-on-catch-all-clauses) in the module docs\n> for further information.","ref":"WaitForIt.html#cond_wait/2-beware-catch-all-clauses-warning"},{"type":"macro","title":"Options - WaitForIt.cond_wait/2","doc":"See the WaitForIt module documentation for further discussion of these options.\n\n  * `:timeout` - the amount of time to wait (in milliseconds) before giving up\n  * `:pre_wait` - wait for the given number of milliseconds before evaluating conditions for the first time\n  * `:frequency` - the polling frequency (in milliseconds) at which to re-evaluate conditions\n  * `:signal` - disable polling and use a signal of the given name instead","ref":"WaitForIt.html#cond_wait/2-options"},{"type":"macro","title":"Examples - WaitForIt.cond_wait/2","doc":"Trigger an alarm when any sensors go beyond a threshold:\n\n    def sound_the_alarm do\n      WaitForIt.cond_wait timeout: 60_000 * 60 * 24 do\n        read_sensor(:sensor1) > 9 -> IO.puts(\"Alarm: :sensor1 too high!\")\n        read_sensor(:sensor2)   IO.puts(\"Alarm: :sensor2 too low!\")\n        read_sensor(:sensor3)   IO.puts(\"Alarm: :sensor3 below zero!\")\n      else\n        IO.puts(\"All is good...for now.\")\n      end\n\n      # Recursively call to wait for the next sensor readings...\n      sound_the_alarm()\n    end","ref":"WaitForIt.html#cond_wait/2-examples"},{"type":"macro","title":"WaitForIt.cond_wait!/2","doc":"The same as `cond_wait/2` but raises a `WaitForIt.TimeoutError` exception if it fails.","ref":"WaitForIt.html#cond_wait!/2"},{"type":"function","title":"WaitForIt.signal/1","doc":"Send a signal to indicate that any processes waiting on the signal should re-evaluate their\nwaiting conditions.","ref":"WaitForIt.html#signal/1"},{"type":"macro","title":"WaitForIt.wait/2","doc":"Wait until the given `expression` evaluates to a truthy value.\n\nReturns the truthy value that ended the wait, or the last falsy value evaluated if a timeout\noccurred.\n\n> ##","ref":"WaitForIt.html#wait/2"},{"type":"macro","title":"Warning {: .warning} - WaitForIt.wait/2","doc":">\n> The value returned from this macro has changed as of version 2.0.\n>\n> In previous versions, `{:ok, value}` would be returned for the success case, and\n> `{:timeout, timeout_milliseconds}` would be returned for the timeout case.\n>\n> As of version 2.0, the final value of the wait expression is returned directly, which will\n> be a truthy value for the success case and a falsy value for the timeout case. This allows\n> the `wait/2` macro to be used in conditional expressions, such as in `if/2`/`else` expressions,\n> or in assertions in tests.\n>\n> If you are migrating from version 1.x and rely on the return value, you can enable the\n> previous behavior by using the `WaitForIt.V1.wait/2` macro instead.","ref":"WaitForIt.html#wait/2-warning-warning"},{"type":"macro","title":"Options - WaitForIt.wait/2","doc":"See the WaitForIt module documentation for further discussion of these options.\n\n  * `:timeout` - the amount of time to wait (in milliseconds) before giving up\n  * `:pre_wait` - wait for the given number of milliseconds before evaluating conditions for the first time\n  * `:frequency` - the polling frequency (in milliseconds) at which to re-evaluate conditions\n  * `:signal` - disable polling and use a signal of the given name instead","ref":"WaitForIt.html#wait/2-options"},{"type":"macro","title":"Examples - WaitForIt.wait/2","doc":"Wait until the top of the hour:\n\n    WaitForIt.wait Time.utc_now.minute == 0, frequency: 60_000, timeout: 60_000 * 60\n\nWait up to one minute for a particular record to appear in the database:\n\n    if data = WaitForIt.wait Repo.get(Post, 42), frequency: 1000, timeout: :timer.seconds(60) do\n      IO.inspect(data)\n    else\n      IO.puts(\"Gave up after #{timeout} milliseconds\")\n    end\n\nAssert that a database record is created by some asynchronous process:\n\n    do_some_async_work()\n    assert %Post{id: 42} = WaitForIt.wait Repo.get(Post, 42)","ref":"WaitForIt.html#wait/2-examples"},{"type":"macro","title":"WaitForIt.wait!/2","doc":"The same as `wait/2` but raises a `WaitForIt.TimeoutError` exception if it fails.","ref":"WaitForIt.html#wait!/2"},{"type":"type","title":"WaitForIt.wait_expression/0","doc":"Type to represent an expression that can be waited on.","ref":"WaitForIt.html#t:wait_expression/0"},{"type":"type","title":"WaitForIt.wait_opt/0","doc":"Options that can be used to control waiting behavior.","ref":"WaitForIt.html#t:wait_opt/0"},{"type":"type","title":"WaitForIt.wait_opts/0","doc":"Options that can be used to control waiting behavior.\n\nSee `t:wait_opt/0`.","ref":"WaitForIt.html#t:wait_opts/0"},{"type":"protocol","title":"WaitForIt.Waitable","doc":"Protocol used for evaluating waitable expressions against waiting conditions to determine if\nwaiting should continue or halt with a final value.","ref":"WaitForIt.Waitable.html"},{"type":"function","title":"WaitForIt.Waitable.evaluate/2","doc":"Evaluates the waitable expression to provide its value, or to continue to wait.\n\nIt should return `{:halt, value}` if the wait is over and the final value of the waitable\nexpression has been determined, or `{:cont, value}` if waiting should continue.","ref":"WaitForIt.Waitable.html#evaluate/2"},{"type":"function","title":"WaitForIt.Waitable.handle_timeout/3","doc":"Provides the final value of the waitable expression in the event of a timeout.","ref":"WaitForIt.Waitable.html#handle_timeout/3"},{"type":"function","title":"WaitForIt.Waitable.wait_type/1","doc":"","ref":"WaitForIt.Waitable.html#wait_type/1"},{"type":"type","title":"WaitForIt.Waitable.t/0","doc":"All the types that implement this protocol.","ref":"WaitForIt.Waitable.html#t:t/0"},{"type":"type","title":"WaitForIt.Waitable.value/0","doc":"","ref":"WaitForIt.Waitable.html#t:value/0"},{"type":"type","title":"WaitForIt.Waitable.wait_type/0","doc":"","ref":"WaitForIt.Waitable.html#t:wait_type/0"},{"type":"module","title":"WaitForIt.V1","doc":"","ref":"WaitForIt.V1.html"},{"type":"macro","title":"WaitForIt.V1.case_wait/3","doc":"Wait until the given `expression` matches one of the case clauses in the given block.\n\nReturns the value of the matching clause, the value of the optional `else` clause,\nor a tuple of the form `{:timeout, timeout_milliseconds}`.\n\nThe `do` block passed to this macro must be a series of case clauses exactly like a built-in\nElixir `case` expression. Just like a `case` expression, the clauses will attempt to be matched\nfrom top to bottom and the first one that matches will provide the resulting value of the\nexpression. The difference with `case_wait` is that if none of the clauses initially matches it\nwill wait and periodically re-evaluate the clauses until one of them does match or a timeout\noccurs.\n\nAn optional `else` clause may also be used to provide the value in case of a timeout. If an\n`else` clause is provided and a timeout occurs, then the `else` clause will be evaluated and\nthe resulting value of the `else` clause becomes the value of the `case_wait` expression. If no\n`else` clause is provided and a timeout occurs, then the value of the `case_wait` expression is a\ntuple of the form `{:timeout, timeout_milliseconds}`.\n\nThe optional `else` clause may also take the form of match clauses, such as those in a case\nexpression. In this form, the `else` clause can match on the final value of the expression that\nwas evaluated before the timeout occurred. See the examples below for an example of this.","ref":"WaitForIt.V1.html#case_wait/3"},{"type":"macro","title":"Options - WaitForIt.V1.case_wait/3","doc":"See the WaitForIt module documentation for further discussion of these options.\n\n  * `:timeout` - the amount of time to wait (in milliseconds) before giving up\n  * `:frequency` - the polling frequency (in milliseconds) at which to re-evaluate conditions\n  * `:signal` - disable polling and use a condition variable of the given name instead\n  * `:pre_wait` - wait for the given number of milliseconds before evaluating conditions for the first time","ref":"WaitForIt.V1.html#case_wait/3-options"},{"type":"macro","title":"Examples - WaitForIt.V1.case_wait/3","doc":"Wait until queue has at least 5 messages, then return them:\n\n    WaitForIt.case_wait Queue.get_messages(queue), timeout: 30_000, frequency: 100 do\n      messages when length(messages) > 4 -> messages\n    else\n      # If after 30 seconds we still don't have 5 messages, just return the messages we do have.\n      messages -> messages\n    end\n\n  A thermostat that keeps temperature in a small range:\n\n    def thermostat(desired_temperature) do\n      WaitForIt.case_wait get_current_temperature() do\n        temp when temp > desired_temperature + 2 ->\n          turn_on_air_conditioning()\n        temp when temp  \n          turn_on_heat()\n      end\n      thermostat(desired_temperature)\n    end\n\n  Ring the church bells every 15 minutes:\n\n    def church_bell_chimes do\n      count = WaitForIt.case_wait Time.utc_now.minute, frequency: 60_000, timeout: 60_000 * 60 do\n        15 -> 1\n        30 -> 2\n        45 -> 3\n        0 -> 4\n      end\n      IO.puts(String.duplicate(\" ding ding ding dong \", count))\n      church_bell_chimes()\n    end","ref":"WaitForIt.V1.html#case_wait/3-examples"},{"type":"macro","title":"WaitForIt.V1.cond_wait/2","doc":"Wait until one of the expressions in the given block evaluates to a truthy value.\n\nReturns the value corresponding with the matching expression, the value of the optional `else`\nclause, or a tuple of the form `{:timeout, timeout_milliseconds}`.\n\nThe `do` block passed to this macro must be a series of expressions exactly like a built-in\nElixir `cond` expression. Just like a `cond` expression, the embedded expresions will be\nevaluated from top to bottom and the first one that is truthy will provide the resulting value of\nthe expression. The difference with `cond_wait` is that if none of the expressions is initially\ntruthy it will wait and periodically re-evaluate them until one of them becomes truthy or a\ntimeout occurs.\n\nAn optional `else` clause may also be used to provide the value in case of a timeout. If an\n`else` clause is provided and a timeout occurs, then the `else` clause will be evaluated and\nthe resulting value of the `else` clause becomes the value of the `cond_wait` expression. If no\n`else` clause is provided and a timeout occurs, then the value of the `cond_wait` expression is a\ntuple of the form `{:timeout, timeout_milliseconds}`.","ref":"WaitForIt.V1.html#cond_wait/2"},{"type":"macro","title":"Options - WaitForIt.V1.cond_wait/2","doc":"See the WaitForIt module documentation for further discussion of these options.\n\n  * `:timeout` - the amount of time to wait (in milliseconds) before giving up\n  * `:frequency` - the polling frequency (in milliseconds) at which to re-evaluate conditions\n  * `:signal` - disable polling and use a condition variable of the given name instead\n  * `:pre_wait` - wait for the given number of milliseconds before evaluating conditions for the first time","ref":"WaitForIt.V1.html#cond_wait/2-options"},{"type":"macro","title":"Examples - WaitForIt.V1.cond_wait/2","doc":"Trigger an alarm when any sensors go beyond a threshold:\n\n    def sound_the_alarm do\n      WaitForIt.cond_wait timeout: 60_000 * 60 * 24 do\n        read_sensor(:sensor1) > 9 -> IO.puts(\"Alarm: :sensor1 too high!\")\n        read_sensor(:sensor2)   IO.puts(\"Alarm: :sensor2 too low!\")\n        read_sensor(:sensor3)   IO.puts(\"Alarm: :sensor3 below zero!\")\n      else\n        IO.puts(\"All is good...for now.\")\n      end\n      sound_the_alarm()\n    end","ref":"WaitForIt.V1.html#cond_wait/2-examples"},{"type":"macro","title":"WaitForIt.V1.signal/1","doc":"Send a signal to the given condition variable to indicate that any processes waiting on the\ncondition variable should re-evaluate their wait conditions.\n\nThe caller of `signal` must be in the same Elixir module as any waiters on the same condition\nvariable since the module is used as a namespace for condition variables. This is to prevent\naccidental name collisions as well as to enforce good practice for encapsulation.","ref":"WaitForIt.V1.html#signal/1"},{"type":"macro","title":"WaitForIt.V1.wait/2","doc":"Wait until the given `expression` evaluates to a truthy value.\n\nReturns `{:ok, value}` or `{:timeout, timeout_milliseconds}`.","ref":"WaitForIt.V1.html#wait/2"},{"type":"macro","title":"Options - WaitForIt.V1.wait/2","doc":"See the WaitForIt module documentation for further discussion of these options.\n\n  * `:timeout` - the amount of time to wait (in milliseconds) before giving up\n  * `:frequency` - the polling frequency (in milliseconds) at which to re-evaluate conditions\n  * `:signal` - disable polling and use a condition variable of the given name instead\n  * `:pre_wait` - wait for the given number of milliseconds before evaluating conditions for the first time","ref":"WaitForIt.V1.html#wait/2-options"},{"type":"macro","title":"Examples - WaitForIt.V1.wait/2","doc":"Wait until the top of the hour:\n\n    WaitForIt.wait Time.utc_now.minute == 0, frequency: 60_000, timeout: 60_000 * 60\n\n  Wait up to one minute for a particular record to appear in the database:\n\n    case WaitForIt.wait Repo.get(Post, 42), frequency: 1000, timeout: 60_000 do\n      {:ok, data} -> IO.inspect(data)\n      {:timeout, timeout} -> IO.puts(\"Gave up after #{timeout} milliseconds\")\n    end","ref":"WaitForIt.V1.html#wait/2-examples"},{"type":"macro","title":"WaitForIt.V1.wait!/2","doc":"Wait until the given `expression` evaluates to a truthy value.\n\nReturns the truthy value or raises a `WaitForIt.TimeoutError` if a timeout occurs.","ref":"WaitForIt.V1.html#wait!/2"},{"type":"macro","title":"Options - WaitForIt.V1.wait!/2","doc":"See the WaitForIt module documentation for further discussion of these options.\n\n  * `:timeout` - the amount of time to wait (in milliseconds) before giving up\n  * `:frequency` - the polling frequency (in milliseconds) at which to re-evaluate conditions\n  * `:signal` - disable polling and use a condition variable of the given name instead\n  * `:pre_wait` - wait for the given number of milliseconds before evaluating conditions for the first time","ref":"WaitForIt.V1.html#wait!/2-options"},{"type":"exception","title":"WaitForIt.TimeoutError","doc":"Exception type to represent a timeout that occurred while waiting.","ref":"WaitForIt.TimeoutError.html"},{"type":"type","title":"WaitForIt.TimeoutError.env/0","doc":"Type to represent the `:env` field of `WaitForIt.TimeoutError` exceptions.\n\nThis struct is a subset of of `Macro.Env` and contains the following fields:\n\n  * `context` - the context of the environment; it can be nil (default context), :guard\n    (inside a guard) or :match (inside a match)\n  * `context_modules` - a list of modules defined in the current context\n  * `file` - the current absolute file name as a binary\n  * `function` - a tuple as {atom, integer}, where the first element is the function name and\n    the second its arity; returns nil if not inside a function\n  * `line` - the current line as an integer\n  * `module` - the current module name","ref":"WaitForIt.TimeoutError.html#t:env/0"},{"type":"type","title":"WaitForIt.TimeoutError.t/0","doc":"","ref":"WaitForIt.TimeoutError.html#t:t/0"},{"type":"extras","title":"Changelog","doc":"# Changelog\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](http://keepachangelog.com/en/1.0.0/)\nand this project adheres to [Semantic Versioning](http://semver.org/spec/v2.0.0.html).","ref":"changelog.html"},{"type":"extras","title":"2.1.0 - 2023--11-14 - Changelog","doc":"#","ref":"changelog.html#2-1-0-2023-11-14"},{"type":"extras","title":"Changed - Changelog","doc":"- Further improved documentation.\n- `WaitForIt.case_wait/3` will now raise a `CaseClauseError` on timeout if there is no `else` block.\n- `WaitForIt.cond_wait/2` will now raise a `CondClauseError` on timeout if there is no `else` block.","ref":"changelog.html#changed"},{"type":"extras","title":"2.0.0 - 2023-11-02 - Changelog","doc":"#","ref":"changelog.html#2-0-0-2023-11-02"},{"type":"extras","title":"Changed - Changelog","doc":"- Much improved documentation.\n- Breaking change to return value of `WaitForIt.wait/2`, `WaitForIt.case_wait/3`, and `WaitForIt.cond_wait/2`.\n- Rewrite of WaitForIt internals.\n- Moved legacy code to `WaitForIt.V1`.","ref":"changelog.html#changed"},{"type":"extras","title":"1.4.0 - 2023-10-24 - Changelog","doc":"#","ref":"changelog.html#1-4-0-2023-10-24"},{"type":"extras","title":"Added - Changelog","doc":"- Add WaitForIt.wait! macro.\n\n## [1.3.0] - 2020-04-02\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Use DynamicSupervisor to manage condition variables.\n\n## [1.2.1] - 2019-03-14\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Add `:pre_wait` option to all forms of waiting.\n\n## [1.2.0] - 2019-03-08\n#","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"- Add support for match clauses in `else` block of `case_wait`. [(Issue #9)](https://github.com/jvoegele/wait_for_it/issues/9)\n\n## [1.1.1] - 2018-03-03\n#","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"- Add idle timeout feature for ConditionVariable.\n\n## [1.1.0] - 2017-09-02\n#","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"- Add support for `else` clause in `case_wait` and `cond_wait`. [(Issue #4)](https://github.com/jvoegele/wait_for_it/issues/4)\n- Add this CHANGELOG\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Use supervisor to manage condition variables. [(Issue #5)](https://github.com/jvoegele/wait_for_it/issues/5)\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Grammar fixes for README and @moduledoc. Thanks to @GregMefford for the fixes.\n- Fix [unexpected messages from wait_for_it when used with Genserver](https://github.com/jvoegele/wait_for_it/issues/3)\n\n## [1.0.0] - 2017-08-28\n- Initial release supporting `wait`, `case_wait`, and `cond_wait` with either polling or condition variable signaling.\n\n[Unreleased]: https://github.com/jvoegele/wait_for_it/compare/v1.1.0...HEAD\n[1.1.1]: https://github.com/jvoegele/wait_for_it/compare/v1.1.0...v1.1.1\n[1.1.0]: https://github.com/jvoegele/wait_for_it/compare/v1.0.0...v1.1.0\n[1.0.0]: https://github.com/jvoegele/wait_for_it/compare/init...v1.0.0","ref":"changelog.html#fixed"},{"type":"extras","title":"License","doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS\n\n   APPENDIX: How to apply the Apache License to your work.\n\n      To apply the Apache License to your work, attach the following\n      boilerplate notice, with the fields enclosed by brackets \"{}\"\n      replaced with your own identifying information. (Don't include\n      the brackets!)  The text should be enclosed in the appropriate\n      comment syntax for the file format. We also recommend that a\n      file or class name and description of purpose be included on the\n      same \"printed page\" as the copyright notice for easier\n      identification within third-party archives.\n\n   Copyright {yyyy} {name of copyright owner}\n\n   Licensed under the Apache License, Version 2.0 (the \"License\");\n   you may not use this file except in compliance with the License.\n   You may obtain a copy of the License at\n\n       http://www.apache.org/licenses/LICENSE-2.0\n\n   Unless required by applicable law or agreed to in writing, software\n   distributed under the License is distributed on an \"AS IS\" BASIS,\n   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n   See the License for the specific language governing permissions and\n   limitations under the License.","ref":"license.html"}],"content_type":"text/markdown"}